from pathlib import Path
from typing import Any

from hatchling.builders.config import BuilderConfig
from hatchling.builders.hooks.plugin.interface import BuildHookInterface
from hatchling.plugin import hookimpl

from hatch_una import util


class UnaBuildHook(BuildHookInterface[BuilderConfig]):
    PLUGIN_NAME = "una-build"

    def initialize(self, version: str, build_data: dict[str, Any]) -> None:
        print("una: Injecting internal dependencies")
        path = Path(self.root)
        conf = util.load_conf(path)

        root_path = path.parents[1]
        extra_root_path = util.EXTRA_PYPROJ / "root"
        if (root_path / util.PYPROJ).exists():
            use_root_path = root_path
        elif (extra_root_path / util.PYPROJ).exists():
            use_root_path = extra_root_path
        else:
            raise ValueError("No root pyproject to determine workspace style")
        root_conf = util.load_conf(use_root_path)
        style: str = root_conf["tool"]["una"]["style"]

        int_deps: dict[str, str] = conf["tool"]["una"]["libs"]
        found = [Path(k) for k in int_deps if (path / k).exists()]
        if not int_deps or not found:
            # should I raise here?
            return

        add_root_pyproj = {str(root_path / util.PYPROJ): str(util.EXTRA_PYPROJ / "root" / util.PYPROJ)}
        if style == "packages":
            add_packages_pyproj = {
                str(f.parents[1] / util.PYPROJ): str(util.EXTRA_PYPROJ / f.name / util.PYPROJ) for f in found
            }
        else:
            add_packages_pyproj = {}

        build_data["force_include"] = {
            **build_data["force_include"],
            **int_deps,
            **add_root_pyproj,
            **add_packages_pyproj,
        }


@hookimpl
def hatch_register_build_hook() -> type[UnaBuildHook]:
    return UnaBuildHook



from pathlib import Path
from typing import Any

from hatchling.metadata.plugin.interface import MetadataHookInterface
from hatchling.plugin import hookimpl

from hatch_una import util


class UnaMetaHook(MetadataHookInterface):
    PLUGIN_NAME = "una-meta"

    def update(self, metadata: dict[str, Any]) -> None:
        print("una: Injecting transitive external dependencies")
        root = Path(self.root)
        conf = util.load_conf(root)
        int_deps: dict[str, str] = conf["tool"]["una"]["libs"]

        project_deps: list[str] = metadata.get("dependencies", {})
        project_deps = [d.strip().replace(" ", "") for d in project_deps]

        add_deps: list[str] = []
        for dep_path in int_deps:
            dep_project_path = Path(dep_path).parents[1]
            extra_path = util.EXTRA_PYPROJ / Path(dep_path).name
            if dep_project_path.exists():
                use_path = dep_project_path
            elif extra_path.exists():
                use_path = extra_path
            else:
                # should I raise here?
                continue
            dep_conf = util.load_conf(use_path)
            dep_deps: list[str] = dep_conf["project"]["dependencies"]
            dep_deps = [d.strip().replace(" ", "") for d in dep_deps]
            add_deps.extend(dep_deps)

        metadata["dependencies"] = list(set(project_deps + add_deps))


@hookimpl
def hatch_register_metadata_hook() -> type[UnaMetaHook]:
    return UnaMetaHook
